from pandas import DataFrame
from tcegoframework.preprocessing.encoders import encode_AD
from tcegoframework.preprocessing.transform import data_preparation


def preprocessing_training_AD(data: DataFrame) -> tuple[DataFrame, DataFrame]:
    cat_col = [
        # 'exercicio_do_orcamento_ano',
        # 'empenho_sequencial_empenho',
        'orgao',
        'orgao_sucessor_atual',
        'tipo_administracao_nome',
        'tipo_poder_nome',
        # 'classificacao_orcamentaria_descricao',
        'funcao',
        'subfuncao',
        'programa',
        'acao',
        'grupo_despesa',
        'elemento_despesa',
        'natureza_despesa_cod',
        'natureza_despesa_nome',
        'formalidade_nome',
        'modalidade_licitacao_nome',
        # 'fonte_recurso_cod',
        'fonte_recurso_nome',
        'beneficiario_cnpj',
        'beneficiario_cpf',
        'beneficiario_cpf/cnpj',
        'periodo',
        'empenho_numero_do_processo',
        # 'empenho_sequencial_empenho.1',
    ]
    text_col = [
        # 'beneficiario_nome',
        'empenho_historico',
    ]
    num_col = [
        'valor_empenhado',
        'valor_anulacao_empenho',
        # 'valor_estorno_anulacao_empenho',
        'valor_cancelamento_empenho',
        # 'valor_anulacao_cancelamento_empenho',
        'valor_saldo_do_empenho',
        'valor_liquidacao_empenho',
        'valor_anulacao_liquidacao_empenho',
        'valor_saldo_liquidado',
        'valor_ordem_de_pagamento',
        'valor_guia_recolhimento',
        'valor_anulacao_ordem_de_pagamento',
        'valor_estorno_anulacao_o_pagamento',
        'valor_estorno_guia_recolhimento',
        'valor_saldo_pago',
        'valor_saldo_a_pagar',
        'valor_a_liquidar',
        'valor_a_pagar_liquidado'
    ]

    data = data.loc[:, (*cat_col,
                        *text_col,
                        *num_col,)]

    X, categorical_columns, numerical_columns = data_preparation(
        data,
        categorical_columns=cat_col,
        numerical_columns=num_col, )

    X = encode_AD(
        X,
        numerical_columns,
        categorical_columns,
        text_col,
        prefix=f'AD',
    )

    return X



import time
from functools import partial

from numpy import array
from pandas.core.frame import DataFrame
from sklearn.metrics import classification_report
from sklearn.model_selection import train_test_split
from tcegoframework import config
from tcegoframework.cfgparsing import get_algorithm, get_validated_data_path
from tcegoframework.flows.inference import (inference_bert_rf_natureza,
                                            inference_corretude,
                                            inference_rf_natureza,
                                            inference_svm_natureza)
from tcegoframework.io import load_excel_data
from tcegoframework.model.metrics import classification_report_csv
from tcegoframework.preprocessing.text import regularize_columns_name


def evaluation_flow():
    # Executing query
    print('Preparando e executando avaliação...')
    data = load_excel_data(get_validated_data_path())
    data = data.reset_index(drop=True)
    data = regularize_columns_name(data)

    if get_algorithm() == 'svm':
        inference_natureza = partial(inference_svm_natureza)
    elif get_algorithm() == 'rf':
        inference_natureza = partial(inference_rf_natureza)
    elif get_algorithm() == 'bert_rf':
        inference_natureza = partial(inference_bert_rf_natureza)

    for label in ['OK', 'INCONCLUSIVO', 'INCORRETO']:
        temp_data = data.loc[data.analise == label]

        print(f'Inferência de Natureza para documentos {label}...')
        time_ref = time.time()
        y_pred_natureza = inference_natureza(temp_data.copy())

        y_true = temp_data.natureza_despesa_cod

        report = classification_report(
            y_true, y_pred_natureza, output_dict=True)
        report = DataFrame(report).transpose()
        report.to_csv(f'{label}_eval_report.csv')
        print(f'Duração total: {(time.time() - time_ref)/60}')

    print('Finalizado.')


def training_evaluation_flow():
    # Executing query
    print('Preparando e executando avaliação...')
    data = load_excel_data(get_validated_data_path())
    data = data.reset_index(drop=True)
    data = regularize_columns_name(data)

    _, data = train_test_split(
        data,
        test_size=0.2,
        random_state=config.RANDOM_SEED,
        stratify=data.analise)

    if get_algorithm() == 'svm':
        inference_natureza = partial(inference_svm_natureza)
    elif get_algorithm() == 'rf':
        inference_natureza = partial(inference_rf_natureza)
    elif get_algorithm() == 'bert_rf':
        inference_natureza = partial(inference_bert_rf_natureza)

    print('Avaliação do classificador de naturezas...')

    for label in ['OK', 'INCONCLUSIVO', 'INCORRETO']:
        temp_data = data.loc[data.analise == label]

        print(f'Inferência de Natureza para documentos {label}...')
        time_ref = time.time()
        y_pred_natureza = inference_natureza(temp_data.copy())

        y_true = temp_data.natureza_despesa_cod

        classification_report_csv(
            y_true, y_pred_natureza, False, f'natureza_{label}_eval_report.csv')
        print(f'Duração total: {(time.time() - time_ref)/60}')

    print('Avaliação do classificador de corretude...')
    time_ref = time.time()
    y_pred = inference_corretude(data.copy())
    y_true = data.analise
    classification_report_csv(
        y_true, y_pred, False, 'corretude_eval_report.csv')
    print(f'Duração total: {(time.time() - time_ref)/60}')

    print('Finalizado.')
